
from django.db.models import F

from exchange.utils import get_price_factory
from exchange.constants import CURRENCY_NAMES, CURRENCIES
from exchange.models import ExchangeRates, _MULTI_CURRENCY_MODELS


def update_all_prices():

    rates = ExchangeRates.get_solo()

    field = lambda c: 'price_{}'.format(CURRENCY_NAMES[c])

    for src, src_name in CURRENCIES:

        params = {}

        for dst, dst_name in CURRENCIES:
            price_factory = get_price_factory(rates, src, dst)

            params[field(dst)] = price_factory(F('price_retail'))

        for model in _MULTI_CURRENCY_MODELS:

            queryset = model.objects.filter(initial_currency=src)

            queryset.update(
                price_uah=0,
                price_usd=0,
                price_eur=0
            )

            queryset.filter(
                price_retail__gt=0
            ).update(
                **params
            )




from django.shortcuts import redirect, HttpResponse
from django.views.decorators.http import require_POST

from exchange.constants import CURRENCY_SESSION_KEY
from exchange.forms import CurrencyForm


@require_POST
def set_currency(request):

    form = CurrencyForm(request.POST)

    if not form.is_valid():
        return HttpResponse(str(form.errors))

    currency = form.cleaned_data['currency']

    request.session[CURRENCY_SESSION_KEY] = currency

    return redirect(request.GET.get('next', 'home'))



from django.contrib import admin

from solo.admin import SingletonModelAdmin

from exchange.models import ExchangeRates
from exchange.tasks import update_all_prices


@admin.register(ExchangeRates)
class ExchangeRatesAdmin(SingletonModelAdmin):

    list_display = ['id', 'usd', 'eur']
    list_editable = ['usd', 'eur']

    def save_model(self, *args):
        super().save_model(*args)
        update_all_prices()




from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


def setup_settings(settings, is_prod, **kwargs):

    if 'solo' not in settings['INSTALLED_APPS']:
        settings['INSTALLED_APPS'] += ['solo']

    for template in settings['TEMPLATES']:
        template['OPTIONS']['context_processors'].append(
            'exchange.context_processors.currencies')


class ExchangeAppConfig(AppConfig):

    name = 'exchange'
    verbose_name = _('Exchange')


default_app_config = 'exchange.ExchangeAppConfig'




from django import forms
from django.db import transaction
from django.contrib import messages
from django.shortcuts import render, redirect
from django.contrib import admin
from django.utils.translation import gettext_lazy as _


class UpdatePricesForm(forms.Form):

    value = forms.IntegerField(label=_('Value'))

    type = forms.ChoiceField(
        label=_('Type'),
        choices=(
            ('percent', _('Percent %')),
            ('number', _('Number')),
        ))


@transaction.atomic
def update_prices(self, request, queryset):

    apply = 'apply' in request.POST

    form = UpdatePricesForm(request.POST if apply else None)

    if apply and form.is_valid():

        value = form.cleaned_data['value']
        is_percent = form.cleaned_data['type'] == 'percent'

        for product in queryset:
            product.calculate_retail_price(is_percent, value)
            product.save()

        messages.success(request, _('Prices updated'))
        return redirect(request.get_full_path())

    context = admin.site.each_context(request)
    context.update({
        'action_name': 'update_prices',
        'form': form,
        'object_list': queryset
    })

    return render(request, 'exchange/update_prices.html', context)


update_prices.short_description = _('Update prices')
